<?php
#
#   authenticate.inc, prepaid accounting, authentication procedure
#   Copyright (C) 2001  James Cameron (quozl@us.netrek.org)
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Procedure
#
#   1) if it is the coordinator's workstation (ws0)
#      require administration password authentication
#      let them do anything once authenticated
#      permit any access to view.phtml
#
#   2) if it is a public workstation (ws1-ws6)
#      find the username using "who"
#      reject access if not logged in
#      allow account viewing only
#
#   3) otherwise reject outright
#
#   Future enhancement expected: accept and validate a username and
#   password combination using the local FTP server as the
#   authentication source, so that centre users can dial-up from home
#   to query their account.
#

# obtain ip patterns for user classes
include('/var/ppa/ip');

# force web browser level authentication
function authenticate($message) {
    header("WWW-Authenticate: Basic realm=\"Gilgandra Internet Centre Accounting\"");
    header("HTTP/1.0 401 Unauthorized");
    echo $message.'<p>Our system has decided not to allow you to access
this web page.<p>Speak with the person in charge if you expect this to work.';
}

function dns($addr, $name) {
    global $SERVER_NAME;

    echo '<table>
<tr><th>Your IP address:</th><td>'.$addr.'</td></tr>
<tr><th>Your DNS name:</th><td>'.$name.'</td></tr>
<tr><th>Our server name:</th><td>'.$SERVER_NAME.'</td></tr>
</table>';
}

# obtain the IP address of the client
$them = gethostbyaddr($REMOTE_ADDR);
# ??? squid proxy may be in use, if so: use a different variable?

# test for IP address in user group
if (!isset($ips[$them])) {
    echo 'Access denied: IP address not known.<p>';
    dns($REMOTE_ADDR, $them);
    echo $ip_users;
    exit;
}

if ($ips[$them] == 'u') {

    # test for non-view URLs
    if (basename($SCRIPT_NAME) != 'view.phtml') {
	echo 'Access denied: URI outside bounds.<p>';
	dns($REMOTE_ADDR, $them);
	exit;
    }

    # determine user by finding their host in the "who" command
    list($host, $domain) = explode('.', $them);
    $pp = popen('who|grep -i '.$host, 'r');
    list($user) = split('  *', chop(fgets($pp,128)));
    pclose($pp);
    if ($user == '') {
	echo 'Access denied: your username could not be determined from the list of currently logged in users. Verify you are logged in.<p>';
	dns($REMOTE_ADDR, $them);
	exit;
    }

    # force view.phtml to take only the user we identified
    $name = $user;

} else

# test for IP address in admin group
if ($ips[$them] == 'a') {

    # test for non-view URLs
    if (basename($SCRIPT_NAME) != 'view.phtml') {

	if(!isset($PHP_AUTH_USER)) {
	    authenticate('Access denied: no username or password provided.');
	    exit;
	} else {
	    $user = strtolower($PHP_AUTH_USER);

	    # obtain encoded passwords from file
	    include('/var/ppa/access');

	    # check if user known
	    if (!isset($admins[$user])) {
		authenticate('Access denied: user '.$user.' not known.');
		exit;
	    }
	    
	    $pass = strtolower($PHP_AUTH_PW);
	    if ($admins[$user] != md5($user.'|'.$pass)) {
		authenticate('Access denied: password incorrect.');
		exit;
	    }
	}
    }

}
?>
